"""
This file is part of hlrc

Copyright(c) sschulz <AT> techfak.uni-bielefeld.de
http://opensource.cit-ec.de/projects/hlrc

This file may be licensed under the terms of the
GNU General Public License Version 3 (the ``GPL''),
or (at your option) any later version.

Software distributed under the License is distributed
on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
express or implied. See the GPL for the specific language
governing rights and limitations.

You should have received a copy of the GPL along with this
program. If not, go to http://www.gnu.org/licenses/gpl.html
or write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

The development of this software was supported by the
Excellence Cluster EXC 277 Cognitive Interaction Technology.
The Excellence Cluster EXC 277 is a grant of the Deutsche
Forschungsgemeinschaft (DFG) in the context of the German
Excellence Initiative.
"""

import logging
from MiddlewareRSB import *

class RobotController:
	def __init__(self, mw_name, scope, loglevel=logging.WARNING):
		self.logger = logging.getLogger(__name__)
		
		# create nice and actually usable formatter and add it to the handler
		self.config_logger(loglevel)
		
		#store scope
		self.scope = scope
		
		if (mw_name.upper() == "RSB"):
			self.logger.info("creating new middleware connection via RSB")
			self.middleware = MiddlewareRSB(self.scope)
		else:
			self.logger.error("invalid middleware requested (%s). supported: {ROS, RSB}\n\n" % (mw_name))
			sys.exit(errno.EINVAL)
	
	def config_logger(self, level):
		formatter = logging.Formatter('%(asctime)s %(name)-30s %(levelname)-8s > %(message)s')
		ch = logging.StreamHandler()
		#ch.setLevel(level)
		ch.setFormatter(formatter)
		self.logger.setLevel(level)
		self.logger.addHandler(ch)

	def set_current_emotion(self, robot_emotion, blocking = False):
		self.logger.debug("set_current_emotion(%s) %s" % (robot_emotion, ("BLOCKING" if blocking else "NON_BLOCKING")))
		self.middleware.set_current_emotion(robot_emotion, blocking)
		
	def set_default_emotion(self, robot_emotion, blocking = False):
		self.logger.debug("set_default_emotion(%s) %s" % (robot_emotion, ("BLOCKING" if blocking else "NON_BLOCKING")))
		self.middleware.set_default_emotion(robot_emotion, blocking)
	
	def set_gaze_target(self, robot_gaze, blocking = False):
		self.logger.debug("set_gaze_target(%s) %s" % (robot_gaze, ("BLOCKING" if blocking else "NON_BLOCKING")))
		self.middleware.set_gaze_target(robot_gaze, blocking)
	
	def set_mouth_target(self, robot_mouth, blocking = False):
		self.logger.debug("set_mouth_target(%s) %s" % (robot_mouth, ("BLOCKING" if blocking else "NON_BLOCKING")))
		self.middleware.set_mouth_target(robot_mouth, blocking)
	
	def set_head_animation(self, robot_animation, blocking = False):
		self.logger.debug("set_head_animation(%s) %s" % (robot_animation, ("BLOCKING" if blocking else "NON_BLOCKING")))
		self.middleware.set_head_animation(robot_animation, blocking)
	
	def set_speak(self, text, blocking = False):
		self.logger.debug("set_speak(%s) %s" % (text, ("BLOCKING" if blocking else "NON_BLOCKING")))
		self.middleware.set_speak(text, blocking)

	def get_gaze_target(self):
		result = self.middleware.get_gaze_target()
		self.logger.debug("get_gaze_target() returned %s" % (result))
		return self.middleware.get_gaze_target()
