#!/usr/bin/python
#PYTHONPATH="/opt/ros/groovy/lib/python2.7/dist-packages:/vol/csra/releases/nightly/lib/python2.7/:/vol/csra/releases/nightly/lib/python2.7/site-packages/
import sys
import logging
import errno

try:
    import rsb
    import rsb.converter
    import rst
    import rstsandbox
    from rst.robot.Animation_pb2 import Animation
except ImportError as exception:
    sys.stderr.write("ImportError: {}\n> HINT: try to export PYTHONPATH=$PYTHONPATH:$YOUR_PREFIX/lib/python2.7/site-packages/\n\n".format(exception))
    sys.exit(errno.ENOPKG)

class hlrc_animation():
    def __init__(self, _base_scope):
        #print "> registering rst converter"
        converter = rsb.converter.ProtocolBufferConverter(messageClass = Animation)
        rsb.converter.registerGlobalConverter(converter)
        self.server = None	
        self.set_scope(_base_scope);

    def __del__(self):
        if (not self.server is None):
            self.server.deactivate()

    def set_scope(self, scope):
        self.base_scope = str(scope) #NOTE: str() is important here, scope is a qstring (?) and gets deleted during call
        print "> setting scope to '%s'" % self.base_scope
	if (not self.server is None):
            self.server.deactivate()
	try:
            self.server = rsb.createRemoteServer(self.base_scope + '/set')
        except ValueError:
            print "> invalid scope given. server deactivated"
            self.server.deactivate()

    def trigger_animation(self, ani_id, repetitions, duration_each, scale, blocking):
        if (self.server is None):
            print("> invalid server")
            return
        #create animation & fill it with values:
        ani = Animation()

        #select ani
        ani.target = ani_id
        ani.repetitions = repetitions
        ani.duration_each = duration_each
        ani.scale       = scale

        if (blocking):
            #blocking:
            print "> calling the animation rpc (blocking until we finished talking)..."
            print '> server reply: "%s"' % self.server.animation(ani)
        else:
            print "> calling the animation rpc (NON-BLOCKING)..."
            return self.server.animation.async(ani)
            #we can block here for a incoming result with a timeout in s
            #print '> server reply: "%s"' % future.get(timeout = 10);

        print "> blocking call done"
    
def main():
    if (len(sys.argv) != 5):
        print "> usage: %s <base scope> <animation id> <dur each> <repetitions>\n>     example: %s /flobi1 1 1000 1" % (sys.argv[0] , sys.argv[0])
        sys.exit(0)

    # Pacify logger.
    #logging.basicConfig()
    scope  = sys.argv[1]
    ani_id = int(sys.argv[2])
    dur    = int(sys.argv[3])
    rep    = int(sys.argv[4])
    scale  = 1.0

    hani = hlrc_animation(scope)
    hani.trigger_animation(ani_id, rep, dur, scale, 1)

if __name__ == '__main__':
    main()


