#!/usr/bin/python
#PYTHONPATH="/opt/ros/groovy/lib/python2.7/dist-packages:/vol/csra/releases/nightly/lib/python2.7/:/vol/csra/releases/nightly/lib/python2.7/site-packages/
import sys
import logging
import errno

try:
    import rsb
    import rsb.converter
    import rst
    import rstsandbox
    from rst.robot.EmotionState_pb2 import EmotionState
except ImportError as exception:
    sys.stderr.write("ImportError: {}\n> HINT: try to export PYTHONPATH=$PYTHONPATH:$YOUR_PREFIX/lib/python2.7/site-packages/\n\n".format(exception))
    sys.exit(errno.ENOPKG)

class hlrc_emotion():
    def __init__(self, _base_scope):
        #print "> registering rst converter"
        converter = rsb.converter.ProtocolBufferConverter(messageClass = EmotionState)
        rsb.converter.registerGlobalConverter(converter)
	self.set_scope( _base_scope);

    def set_scope(self, scope):
        self.base_scope = str(scope) #NOTE: str() is important here, scope is a qstring (?) and gets deleted during call
        print "> setting scope to '%s'" % self.base_scope
        try:
            self.server = rsb.createRemoteServer(self.base_scope + '/set')
        except ValueError:
            print "> invalid scope given. server deactivated"
            self.server.deactivate()

    def set_emotion(self, emotion_id, duration_each, blocking):
        if (self.server is None):
            print("> invalid server")
            return

        #create emotion & fill it with values:
        em = EmotionState()

        #select ani
        em.value    = emotion_id
        em.duration = duration_each

        with rsb.createRemoteServer(self.base_scope + '/set') as server:
            if (blocking):
                #blocking:
                print "> calling the emotion rpc (blocking until we finished talking)..."
                print '> server reply: "%s"' % server.currentEmotion(em)
            else:
               print "> calling the animation rpc (NON-BLOCKING)..."
               future = server.currentEmotion.async(em)
               #we can block here for a incoming result with a timeout in s
               #print '> server reply: "%s"' % future.get(timeout = 10);

        print "> done"
    
def main():
    if (len(sys.argv) != 4):
        print "> usage: %s <base scope> <emotion id> <duration>\n>     example: %s /flobi1 1 1000" % (sys.argv[0] , sys.argv[0])
        sys.exit(0)

    # Pacify logger.
    #logging.basicConfig()
    em_id = int(sys.argv[2])
    dur    = int(sys.argv[3])
    base   = sys.argv[1]

    hlrc_emotion(base).set_emotion(em_id, dur, 1)

if __name__ == '__main__':
    main()


