#!/usr/bin/python
import sys
import os.path
from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import SIGNAL
from hlrc_speak_utterance import hlrc_utterance
from hlrc_play_animation import hlrc_animation
from hlrc_set_emotion import hlrc_emotion

class Example(QtGui.QWidget):
    
    def __init__(self, _scope):
        super(Example, self).__init__()
        self.scope = _scope
        self.initUI()
	scope = self.scope_lineedit.text()
        self.hlrc_utterance = hlrc_utterance(scope)
        self.hlrc_animation = hlrc_animation(scope)
        self.hlrc_emotion   = hlrc_emotion(scope)

    def scopeChanged(self):
	self.scope = self.scope_lineedit.text()
        print "> changed scope to %s" % self.scope
        self.hlrc_utterance.set_scope(self.scope)
        self.hlrc_animation.set_scope(self.scope)
        self.hlrc_emotion.set_scope(self.scope)

    def initUtteranceUI(self):
        groupBox = QtGui.QGroupBox("utterances")

        hello = QtGui.QPushButton("hello...")
        hello.clicked.connect(self.utterance_button_clicked)

        count = QtGui.QPushButton("12345...")
        count.clicked.connect(self.utterance_button_clicked)


        vbox = QtGui.QVBoxLayout()
        vbox.addWidget(hello)
        vbox.addWidget(count)
        vbox.addStretch(1)
        groupBox.setLayout(vbox)

        return groupBox

    def initSpinbox(self, _min, _max, _init, double):
        if (double): 
            s = QtGui.QDoubleSpinBox()
        else:
            s = QtGui.QSpinBox()

        s.setMinimum(_min)
        s.setMaximum(_max)
        s.setValue(_init)
        return s

    def initEmotionUI(self):
        groupBox = QtGui.QGroupBox("emotion")
        neutral   = QtGui.QPushButton("neutral")
        neutral.clicked.connect(self.emotion_button_clicked)
        surprised = QtGui.QPushButton("surprise")
        surprised.clicked.connect(self.emotion_button_clicked)
        angry     = QtGui.QPushButton("angry")
        angry.clicked.connect(self.emotion_button_clicked)
        happy     = QtGui.QPushButton("happy")
        happy.clicked.connect(self.emotion_button_clicked)
        sad     = QtGui.QPushButton("sad")
        sad.clicked.connect(self.emotion_button_clicked)
        fear      = QtGui.QPushButton("fear")
        fear.clicked.connect(self.emotion_button_clicked)

        self.duration_spinbox = self.initSpinbox(100, 10000, 1000, 0)

        vbox = QtGui.QVBoxLayout()
        vbox.addWidget(self.duration_spinbox)
        vbox.addWidget(neutral)
        vbox.addWidget(surprised)
        vbox.addWidget(angry)
        vbox.addWidget(happy)
        vbox.addWidget(sad)
        vbox.addWidget(fear)
        vbox.addStretch(1)
        groupBox.setLayout(vbox)
        return groupBox


    def initAnimationButton(self, name):
        b = QtGui.QPushButton(name)
        b.clicked.connect(self.animation_button_clicked)
        return b

    def initAnimUI(self):
        groupBox = QtGui.QGroupBox("animations")
  
        #options
        options = QtGui.QHBoxLayout()
        vbox_r = QtGui.QVBoxLayout()
        vbox_r.addWidget(QtGui.QLabel("repetitions"))
        self.repetitions_spinbox = self.initSpinbox(1,10,1,0)
        vbox_r.addWidget(self.repetitions_spinbox)

        vbox_d = QtGui.QVBoxLayout()
        vbox_d.addWidget(QtGui.QLabel("duration/ms (each)"))
        self.duration_each_spinbox = self.initSpinbox(100,10000,1000,0)
        vbox_d.addWidget(self.duration_each_spinbox)

        vbox_s = QtGui.QVBoxLayout()
        vbox_s.addWidget(QtGui.QLabel("scale"))
        self.scale_spinbox = self.initSpinbox(0.1, 5.0, 1.0, 1)
        vbox_s.addWidget(self.scale_spinbox)


        options.addLayout(vbox_r)
        options.addLayout(vbox_d)        
        options.addLayout(vbox_s)

        #buttons
        vbox = QtGui.QVBoxLayout()
        vbox.addLayout(options)
        vbox.addWidget(self.initAnimationButton("head shake"))
        vbox.addWidget(self.initAnimationButton("head nod"))
        vbox.addWidget(self.initAnimationButton("eyeblink left"))
        vbox.addWidget(self.initAnimationButton("eyeblink right"))
        vbox.addWidget(self.initAnimationButton("eyeblink both"))
	vbox.addWidget(self.initAnimationButton("eyebrows raise"))
	vbox.addWidget(self.initAnimationButton("eyebrows lower"))
        vbox.addStretch(1)
        groupBox.setLayout(vbox)

        return groupBox

    def initUI(self):        

        self.scope_lineedit = QtGui.QLineEdit(self.scope);
        self.connect(self.scope_lineedit, SIGNAL("editingFinished()"),self.scopeChanged)

	vbox = QtGui.QVBoxLayout();
	
	hboxl = QtGui.QHBoxLayout();
	hboxl.addWidget(QtGui.QLabel("base scope: "));
	hboxl.addWidget(self.scope_lineedit);
	vbox.addLayout(hboxl);

        hbox = QtGui.QHBoxLayout()
	vbox.addLayout(hbox);
	
        #emotion
        hbox.addWidget(self.initEmotionUI())

        #utterance:
        hbox.addWidget(self.initUtteranceUI())

        #animations:
        hbox.addWidget(self.initAnimUI())

        self.setLayout(vbox)
        #sld.valueChanged.connect(lcd.display)
        
        self.setGeometry(400, 100, 250, 150)
        self.setWindowTitle("HLC Server test GUI")
        self.show()

    def utterance_button_clicked(self):
        button = self.sender().text()
        
        fn_praat = "b.praat"
        fn_wav   = "b.wav"

        if (button == "12345..."):
            print("123")
            fn_praat = "1234.praat"
            fn_wav   = "1234.wav"

        elif (button == "hello..."):
            print "hello"
            fn_praat = "hello_my_name.praat"
            fn_wav   = "hello_my_name.wav"

        else:
            print "invalid button '%s'" % (button)
            return

        #try to find the wav/praat files:
        path = os.path.dirname(os.path.realpath(__file__)) + "/../share/hlrc_server/examples/"
        if (not os.path.isfile(path + fn_wav)):
            #try fallback solution for running from current folder:
            path = "examples/"

        self.hlrc_utterance.trigger_utterance(path + fn_praat, path + fn_wav, 0)

    def emotion_button_clicked(self):
        button = self.sender().text()
        duration = self.duration_spinbox.value()


        if (button == "angry"):
            self.hlrc_emotion.set_emotion(3, duration, 0)
        elif (button == "neutral"):
            self.hlrc_emotion.set_emotion(0, duration, 0)
        elif (button == "happy"):
            self.hlrc_emotion.set_emotion(1, duration, 0)
        elif (button == "sad"):
            self.hlrc_emotion.set_emotion(2, duration, 0)
        elif (button == "surprise"):
            self.hlrc_emotion.set_emotion(4, duration, 0)
        elif (button == "fear"):
            self.hlrc_emotion.set_emotion(5, duration, 0)
        else:
            print "invalid emotion id '%s'" % (button)
            return

    def animation_button_clicked(self):
        button = self.sender().text()

        repetitions = self.repetitions_spinbox.value()
        duration_each = self.duration_each_spinbox.value()
        scale = self.scale_spinbox.value()

        if (button == "head nod"):
            self.hlrc_animation.trigger_animation(1, repetitions, duration_each, scale, 0)
        elif (button == "head shake"):
            self.hlrc_animation.trigger_animation(2, repetitions, duration_each, scale, 0)
        elif (button == "eyeblink left"):
            self.hlrc_animation.trigger_animation(3, repetitions, duration_each, scale, 0)
        elif (button == "eyeblink right"):
            self.hlrc_animation.trigger_animation(4, repetitions, duration_each, scale, 0)
        elif (button == "eyeblink both"):
            self.hlrc_animation.trigger_animation(5, repetitions, duration_each, scale, 0)
        elif (button == "eyebrows raise"):
            self.hlrc_animation.trigger_animation(6, repetitions, duration_each, scale, 0)
        elif (button == "eyebrows lower"):
            self.hlrc_animation.trigger_animation(7, repetitions, duration_each, scale, 0)
        else:
            print "invalid button '%s'" % (button)
            return

def main():
    scope = "/home/wardrobe/flobi"
    if (len(sys.argv) == 2):
	scope = sys.argv[1]

    app = QtGui.QApplication(sys.argv)
    ex = Example(scope)
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()
